;; ARM code analyser
;; Copyright (C) 2006 Tomasz Malesinski <tmal@mimuw.edu.pl>

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(load "arman.lisp")
(load "iriversub.lisp")
(load "iriverumssub.lisp")
(load "iriver5xxsub.lisp")

(defun make-iriver-image (fname1 fname2 &optional fnameiram)
  (let ((im1
	 (list
	  (image-load-block fname1 #x800000 #x87ffff)
	  (image-load-block fname2 #xa00000 #xafffff))))
    (if fnameiram
	(cons
	  (image-load-block fnameiram #x40d000 #x40e693)
	  im1)
      im1)))

(defun make-old-iriver-image (fname1 fname2)
  (list
   (image-load-block fname1 #x8000000 #x805ffff)
   (image-load-block fname2 #xa00000 #xafffff)))

;; (defparameter iriver-image
;;   (make-iriver-image "ifp_1.21_1.bin" "ifp_1.21_2.bin"))
(defparameter iriverums-image
  (make-iriver-image "ifpums129eu1.bin" "ifpums129eu2.bin"
		     "ifpums129euiram.bin"))
(defparameter iriver5xx-image
  (make-old-iriver-image "ifp5xx1.bin" "ifp5xx2.bin"))

;; (defvar backdoor-image
;;   (make-iriver-image "/home/tmal/iriver/backdoor/backdoor1.bin"
;; 		     "/home/tmal/iriver/backdoor/backdoor1.bin"))

(defun iriver-ext-syntax ()
  (subroutines-ext-syntax iriver-image iriver-subroutines))

(defun iriverums-ext-syntax ()
  (subroutines-ext-syntax iriverums-image iriverums-subroutines))

(defun iriver5xx-ext-syntax ()
  (subroutines-ext-syntax iriver5xx-image iriver5xx-subroutines))

(defun disassemble-iriver ()
  (print-disassemble iriver-image #x800000 #x870000)
  (print-disassemble iriver-image #xa00000 #xa10000))

(defun disassemble-iriverums ()
  (print-disassemble iriverums-image #x40d000 #x40e693)
  (print-disassemble iriverums-image #x800000 #x870000)
  (print-disassemble iriverums-image #xa00000 #xa10000))

(defun disassemble-iriver5xx ()
  (print-disassemble iriver5xx-image #x8000000 #x805ffff)
  (print-disassemble iriverums-image #xa00000 #xa10000))

(defun simple-disassemble (file base start end)
  (let ((image (list (image-load-block file base end))))
    (print-disassemble image start end)))

(defun emuriver-functions-table (subroutines version)
  (princ "/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include \"trace.h\"
")
  (format t "struct function functions_~a[] = {~%" version)
  (dolist (sub subroutines)
    (format t "  {0x~x, \"~a\", ~d, ~d},~%"
	    (car sub)
	    (string-downcase (assoc-get 'name (cdr sub)
					(format nil "p~x" (car sub))))
	    (length (assoc-get 'args (cdr sub)))
	    (cond
	     ((assoc-get 'do-not-emulate (cdr sub)) 1)
	     ((assoc-get 'emulate-fold (cdr sub)) 2)
	     (t 0))))
  (format t "  {0, NULL, 0, 0}~%};~%"))

(defun iriver-functions-table ()
  (emuriver-functions-table iriver-subroutines "ifp121"))

(defun iriverums-functions-table ()
  (emuriver-functions-table iriverums-subroutines "ifpums129eu"))

(defun iriverums-find-new ()
  (find-new-subroutines iriverums-image iriverums-subroutines :file "sub.lisp"
			:name "iriverums-subroutines"))
;;(find-new-subroutines iriver-image (mapcar #'(lambda (x) (cons (car x) (cons (cons 'find-sub-new t) (cdr x)))) iriver-subroutines))

(defun iriver5xx-find-new ()
  (find-new-subroutines iriver5xx-image iriver5xx-subroutines :file "sub.lisp"
			:name "iriver5xx-subroutines"))
